# pylint: disable-msg=E1101,W0613,W0603

from __future__ import unicode_literals

import pandas as pd
from pandas.util.decorators import Appender

from pyopendata.base import DataStore, DataResource, _shared_docs


_ckan_doc_kwargs = dict(resource_klass='CKANResource')


class CKANStore(DataStore):
    """
    Storage class to read data via CKAN API

    Parameters
    ----------
    url : string
        URL for CKAN API

    """

    _cache_attrs = ['_datasets', '_packages', '_groups', '_tags']

    @classmethod
    def _normalize_url(cls, url):
        url = DataStore._normalize_url(url)
        if url.endswith('api'):
            # remove '/api'
            url = url[:-4]
        return url

    @Appender(_shared_docs['is_valid'])
    def is_valid(self):
        try:
            response = self._requests_get('/api/action/site_read')
            results = self._validate_response(response)
            return True
        except self._connection_errors:
            return False
        except Exception:
            return False

    def get(self, object_id):
        """
        Get instance related to object_id. First try to get_resource, then get_package.

        Parameters
        ----------
        object_id : str
            id to specify resource or package

        Returns
        -------
        result : CKANResouce or CKANPackage
        """
        try:
            return self.get_resource(object_id)
        except self._connection_errors:
            pass
        try:
            return self.get_package(object_id)
        except self._connection_errors:
            raise

    def get_package(self, package_id):
        """
        Get package related to package_id.

        Parameters
        ----------
        package_id : str
            id to specify package

        Returns
        -------
        result : CKANPackage
        """
        params = dict(id=package_id)
        response = self._requests_get('/api/action/package_show', params=params)
        results = self._validate_response(response)
        return CKANPackage(_store=self, **results)

    @Appender(_shared_docs['get'] % _ckan_doc_kwargs)
    def get_resource(self, resource_id):
        params = dict(id=resource_id)
        response = self._requests_get('/api/action/resource_show', params=params)
        results = self._validate_response(response)
        return CKANResource(_store=self, **results)

    def get_resources_from_tag(self, tag):
        params = dict(id=tag)
        response = self._requests_get('/api/action/tag_show', params=params)
        results = self._validate_response(response)
        results = results['packages']
        return [CKANResource(_store=self, **r) for r in results]

    def get_packages_from_group(self, group_id):
        params = dict(id=group_id)
        response = self._requests_get('/api/action/group_show', params=params)
        results = self._validate_response(response)
        results = results['packages']
        return [CKANPackage(_store=self, **r) for r in results]

    @Appender(_shared_docs['search'] % _ckan_doc_kwargs)
    def search(self, search_string):
        # get smaller object to larger object (resource -> package)
        try:
            return self.search_resource(search_string)
        except self._connection_errors:
            pass
        try:
            return self.search_package(search_string)
        except self._connection_errors:
            raise

    def search_package(self, search_string):
        params = dict(q=search_string)
        response = self._requests_get('/api/action/package_search', params=params)
        results = self._validate_response(response)
        results = results['results']
        return [CKANPackage(_store=self, **r) for r in results]

    def search_resource(self, search_string):
        # different params from search_packages
        # params = dict(query=search_string)
        # response = requests.get('{0}/api/action/resource_search'.format(self.url), params=params)

        # avoid escape search string (:)

        request_url = '/api/action/resource_search?query={0}'.format(search_string)
        response = self._requests_get(request_url)
        results = self._validate_response(response)
        results = results['results']
        return [CKANResource(_store=self, **r) for r in results]

    @property
    def datasets(self):
        return self.packages

    @property
    def packages(self):
        if self._packages is None:
            response = self._requests_get('/api/action/package_list')
            results = self._validate_response(response)
            if isinstance(results, list):
                self._packages = [CKANPackage(_store=self, name=r) for r in results]
                # self._packages = results
            elif isinstance(results, dict):
                # internally calls ``current_package_list_with_resources``?
                results = results['results']
                self._packages = [CKANPackage(_store=self, **r) for r in results]
            else:
                raise ValueError(type(results), results)
        return self._packages

    @property
    def groups(self):
        if self._groups is None:
            response = self._requests_get('/api/action/group_list')
            self._groups = self._validate_response(response)
        return self._groups

    @property
    def tags(self):
        if self._tags is None:
            response = self._requests_get('/api/action/tag_list')
            self._tags= self._validate_response(response)
        return self._tags

    def _validate_response(self, response):
        if response.status_code != 200:
            raise ValueError(response.status_code)

        response_dict = response.json()
        if response_dict['success'] is not True:
            raise valueError(responce_dict['message'])
        return response_dict['result']


class CKANPackage(DataResource):

    def __init__(self, _store=None, resources=None, name=None, **kwargs):
        DataResource.__init__(self, **kwargs)

        if _store is None:
            raise ValueError('_store must be passed')
        else:
            self.store = _store
        if name is None:
            raise ValueError("Package doesn't have 'name' attribute")
        else:
            self.name = name

        if isinstance(resources, list):
            self._resources = [CKANResource(_store=self.store, **r) for r in resources]
        else:
            self._resources = None

    def __unicode__(self):
        source_len = len(self.resources)
        if source_len == 0:
            return '{0} (Empty)'.format(self.name)
        elif source_len == 1:
            return '{0} (1 resource)'.format(self.name)
        else:
            return '{0} ({1} resources)'.format(self.name, source_len)

    @property
    def resources(self):
        # if resources is blank, retrieve from the store
        if self._resources is None:
            self._resources = self.store.get_package(self.name).resources
        return self._resources

    @Appender(_shared_docs['get'] % _ckan_doc_kwargs)
    def get(self, resource_id):
        return self.get_resource(resource_id)

    @Appender(_shared_docs['get'] % _ckan_doc_kwargs)
    def get_resource(self, resource_id):
        for r in self.resources:
            if r.id == resource_id:
                return r
        raise KeyError('Unable to find resource {0} in {1}'.format(resouce_id, self.name))

    def read(self, raw=False, **kwargs):
        """
        Read data from its resource if the number of resource is 1.
        Otherwise, raise ValueError.

        Parameters
        ----------
        raw : bool, default False
            If False, return pandas.DataFrame. If True, return raw data
        kwargs:
            Keywords passed to pandas.read_xxx function

        Returns
        -------
        data : pandas.DataFrame or requests.raw.data
        """
        source_len = len(self.resources)
        if source_len == 0:
            if raw:
                return None
            else:
                return pd.DataFrame()
        elif source_len == 1:
            return self.resources[0].read(raw=raw, **kwargs)
        else:
            raise ValueError('Package has {0} resources. Specify target CKANResource to read'.format(source_len))


class CKANResource(DataResource):
    _attrs = ['size_text']
    _supported_formats = ('CSV', 'JSON', 'XLS')

    def __init__(self, _store=None, resources=None, **kwargs):
        DataResource.__init__(self, **kwargs)

        if _store is None:
            raise ValueError('_store must be passed')
        else:
            self.store = _store

        if resources is None:
            self.resources = None
        elif isinstance(resources, list):
            self.resources = [CKANResource(_store=self.store, **r) for r in resources]
        else:
            raise ValueError(type(resources), resources)

    def __unicode__(self):
        rep_str = """Resource ID: {id}
Resource Name: {name}
Resource URL: {url}
Format: {format}, Size: {size}""".format(id=self.id, name=self.name,
                                         url=self.url,
                                         format=self.format,
                                         size=self.size_text)
        return rep_str

    @Appender(_shared_docs['read'])
    def read(self, raw=False, **kwargs):
        if self.resources is None:
            return DataResource.read(self, raw=raw, **kwargs)
        else:
            source_len = len(self.resources)
            if source_len == 0:
                return pd.DataFrame()
            elif source_len == 1:
                return self.resources[0].read(raw=raw, **kwargs)
            else:
                raise ValueError('Package has {0} resources. Use CKANResource.read()'.format(source_len))

    def _read(self, **kwargs):
        try:
            # try to parse a StringIO first, then URL
            content = self._read_raw()
            return self._read_ext(content, **kwargs)
        except Exception:
            return self._read_ext(self.url, **kwargs)

    def _read_ext(self, target, **kwargs):
        if self.format in ('CSV', 'CSV/TXT'):
            return pd.read_csv(target, **kwargs)
        elif self.format == 'XLS':
            return pd.read_excel(target, **kwargs)
        elif self.format == 'JSON':
            return pd.read_json(target, **kwargs)
        elif self.format == 'N/A':
            raise ValueError('{0} is not available on the store'.format(self.name))
        else:
            raise ValueError('Unsupported read format: {0}'.format(self.format))



# pylint: disable-msg=E1101,W0613,W0603

from __future__ import unicode_literals

import numpy as np
import pandas as pd
from pandas.compat import u, range, iterkeys, iteritems
from pandas.util.decorators import Appender

from pyopendata.base import DataStore, DataResource, _shared_docs
import pyopendata.io.sdmx as sdmx


_eurostat_doc_kwargs = dict(resource_klass='EurostatResource')


class EurostatStore(DataStore):
    """
    Storage class to read Eurostat data
    """
    # http://epp.eurostat.ec.europa.eu/portal/page/portal/sdmx_web_services/getting_started/rest_sdmx_2.1

    _url = 'http://www.ec.europa.eu/eurostat/SDMX/diss-web/rest'
    _cache_attrs = ['_datasets']

    @Appender(_shared_docs['get'] % _eurostat_doc_kwargs)
    def get(self, data_id):
        resource = EurostatResource(id=data_id)
        return resource

    @property
    def datasets(self):
        if self._datasets is None:
            response = self._requests_get('/dataflow/ESTAT/all/latest')
            import xml.etree.cElementTree as ET
            root = ET.fromstring(response.content)

            self._datasets = []
            for dataflow in root.iter(sdmx._STRUCTURE + 'Dataflow'):
                name = sdmx._get_english_name(dataflow)
                id = dataflow.get('id')
                resource = EurostatResource(name=name, id=id)
                self._datasets.append(resource)
        return self._datasets


class EurostatResource(DataResource):

    def __init__(self, **kwargs):
        DataResource.__init__(self, **kwargs)
        self.url = EurostatStore._url + '/data/{0}/?'.format(self.id)
        self.dsd_url = EurostatStore._url + '/datastructure/ESTAT/DSD_{0}'.format(self.id)

        self._dsd = None

    @property
    def dsd(self):
        if self._dsd is None:
            dsd = self._requests_get(url=self.dsd_url).content
            self._dsd = sdmx._read_sdmx_dsd(dsd)
        return self._dsd

    def _read(self, **kwargs):

        try:
            # try to use dsd if available
            dsd = self.dsd
        except Exception:
            dsd = None

        data = self._requests_get().content
        result = sdmx.read_sdmx(data, dsd=dsd)
        # There is data not sorted by time
        result = result.sort_index()
        return result





# pylint: disable-msg=E1101,W0613,W0603

from __future__ import unicode_literals

import copy
import datetime
import json
import warnings

import numpy as np
import pandas as pd
from pandas.compat import StringIO, iterkeys, iteritems
from pandas.util.decorators import Appender

from pyopendata.base import DataStore, DataResource, _shared_docs
from pyopendata.io import read_jsdmx
from pyopendata.util import network


_worldbank_doc_kwargs = dict(resource_klass='WorldBankResource')


class WorldBankStore(DataStore):
    """
    Storage class to read World Bank data
    """
    # http://search.worldbank.org/api/v2/projects

    _url = 'http://api.worldbank.org'
    _cache_attrs = ['_datasets']

    @Appender(_shared_docs['get'] % _worldbank_doc_kwargs)
    def get(self, resource_id):
        query = '/countries/all/indicators/{0}?format=json'.format(resource_id)
        url = self.url + query
        return WorldBankResource(id=resource_id, url=url)


class WorldBankResource(DataResource):

    entries_per_page = 500

    def _read_pagenate(self, **kwargs):
        """Because of pagenation, raw_contet stores parsed json data as it is
        * _read_raw will return it after converting to string
        * _read will convert it to pandas.DataFrame
        """
        if self._raw_content is None:
            page = 1
            contents = []
            pb = None

            while(True):
                query = '&page={0}&per_page={1}'.format(page, self.entries_per_page)
                data = self._requests_get(query).json()
                meta = data[0]
                content = data[1]

                if pb is None:
                    # progressbar cannot be initialized
                    # arter retrieving total number of pages
                    pb = network.ProgressBar(total=meta['pages'])
                pb.update(1)
                contents.extend(content)
                if page >= meta['pages']:
                    break
                page = page + 1

            self._raw_content = contents
        return self._raw_content

    def _read_raw(self, **kwargs):
        contents = copy.deepcopy(self._read_pagenate(**kwargs))
        return StringIO(json.dumps(contents))

    def _read(self, **kwargs):
        # meta, data = json.loads(json_raw)
        page = 1
        contents = []

        def assign_value(entry):
            # each attribute can contain dict as value.
            # In this case, retrieve 'value' from the list
            #
            # {'date': '2009',
            #  'country': {'id': 'L5', 'value': 'Andean Region'},
            #  'indicator': {'id': 'NY.GDP.PCAP.KD.ZG', 'value': 'GDP per capita growth (annual %)'},
            #  'decimal': '1', 'value': None},

            for k, v in iteritems(entry):
                if isinstance(v, dict):
                    entry[k] = v['value']
            if 'date' in entry:
                dt = entry['date']
                if '1900' <= dt <= '2100':
                    entry['date'] = datetime.datetime(int(dt), 1, 1)
            for key in ['decimal', 'value']:
                if key in entry and entry[key] is not None:
                    entry[key] = float(entry[key])
            if entry['decimal'] != 0:
                message = "The entry has 'decimal' != 0, it may not be parsed properly "
                warnings.warn(message , UserWarning)
            return entry
        contents = copy.deepcopy(self._read_pagenate(**kwargs))
        contents = [assign_value(c) for c in contents]
        result = pd.DataFrame(contents)

        try:
            result = pd.pivot_table(result, index='date', columns=['indicator', 'country'],
                                    values='value', aggfunc=np.sum)
        except Exception:
            pass

        return result






# pylint: disable-msg=E1101,W0613,W0603

from __future__ import unicode_literals
from __future__ import division

import os
import sys

import requests
import pandas
from pandas.compat import StringIO, bytes_to_str, binary_type
from pandas.util.decorators import Appender

from pyopendata.util import network

_shared_docs = dict()
_base_doc_kwargs = dict(resource_klass='DataResource')


class DataResource(pandas.core.base.StringMixin):
    """
    Represents a data contained in the URL
    """

    # url being used as default / static
    _url = None
    # kwargs which should be stored as instance properties
    _attrs = []
    # instance properties used as cache (overwritten by inherited classes)
    _cache_attrs = []

    _chunk_size = 1024 * 1024

    def __init__(self, format=None, id=None, name=None, url=None, proxies=None,
                 size=None, **kwargs):

        if isinstance(format, pandas.compat.string_types):
            format = format.strip().upper()

        self.format = format
        self.id = id
        self.name = name
        self.url = self._normalize_url(url)
        self.proxies = proxies
        self.size = size

        for attr in self._attrs:
            value = kwargs.pop(attr, None)
            setattr(self, attr, value)
        # cache for raw content
        self._raw_content = None

        self._initialize_attrs(self)

        self.kwargs = kwargs

    def __unicode__(self):
        return '{0} ({1})'.format(self.__class__.__name__, self.url)

    @classmethod
    def _initialize_attrs(cls, obj):
        """
        Initialize object with class attributes
        """
        # initialize cache attrs
        for attr in cls._cache_attrs:
            setattr(obj, attr, None)
        return obj

    @classmethod
    def _normalize_url(cls, url):
        if url is None:
            return cls._url
        elif url.endswith('/'):
            # remove final slash to handle sitename and filename commonly
            return url[:-1]
        else:
            return url

    def _requests_get(self, action='',  params=None, url=None):
        """
        Internal requests.get to handle proxy
        """
        if url is None:
            url = self.url
        response = requests.get(url + action, params=params, proxies=self.proxies)
        return response

    _shared_docs['read'] = (
        """Read data from resource

        Parameters
        ----------
        raw : bool, default False
            If False, return pandas.DataFrame. If True, return raw data
        kwargs:
            Keywords passed to pandas.read_xxx function

        Returns
        -------
        data : pandas.DataFrame or requests.raw.data

        Notes
        -----
        - Depending on the target format, parsing to ``pandas.DataFrame`` may fail.
          Use ``raw=True`` to get raw data in such cases.
        """)

    @Appender(_shared_docs['read'])
    def read(self, raw=False, **kwargs):
        if raw:
            content = self._read_raw(**kwargs)
            return content.getvalue()
        else:
            return self._read(**kwargs)

    def _read(self):
        raise NotImplementedError

    def _read_raw(self, **kwargs):
        if self._raw_content is None:
            response = self._requests_get()
            content_length = response.headers.get('content-length')
            out = StringIO()

            try:
                content_length = int(content_length)
                pb = network.ProgressBar(total=content_length)

                for chunk in response.iter_content(self._chunk_size):
                    if chunk:
                        out.write(chunk)
                        pb.update(self._chunk_size)
                self._raw_content = out
            except Exception as e:
                # print(e)
                # no content_length or any errors
                if isinstance(response.content, binary_type):
                    out.write(bytes_to_str(response.content))
                else:
                    out.write(response.content)
                self._raw_content = out
        return self._raw_content


class DataStore(DataResource):
    _connection_errors = (requests.exceptions.ConnectionError, ValueError)
    _cache_attrs = ['_datasets']

    def __new__(cls, kind_or_url=None, proxies=None):
        from pyopendata.ckan import CKANStore
        from pyopendata.eurostat import EurostatStore
        from pyopendata.oecd import OECDStore
        from pyopendata.undata import UNdataStore
        from pyopendata.worldbank import WorldBankStore

        if kind_or_url == 'oecd' or cls is OECDStore:
            return OECDStore._initialize(proxies=proxies)
        elif kind_or_url == 'eurostat' or cls is EurostatStore:
            return EurostatStore._initialize(proxies=proxies)
        elif kind_or_url == 'undata' or cls is UNdataStore:
            return UNdataStore._initialize(proxies=proxies)
        elif kind_or_url == 'worldbank' or cls is WorldBankStore:
            return WorldBankStore._initialize(proxies=proxies)

        elif cls is CKANStore:
            # skip validation if initialized with CKANStore directly
            store = CKANStore._initialize(url=kind_or_url, proxies=proxies)
            return store
        else:
            store = CKANStore._initialize(url=kind_or_url, proxies=proxies)
            if store.is_valid():
                return store
        raise ValueError('Unable to initialize DataStore with {0}'.format(kind_or_url))

    def __init__(self, kid_or_url=None, proxies=None):
        # handle with __new__
        pass

    @classmethod
    def _initialize(cls, url=None, proxies=None):
        obj = object.__new__(cls)
        obj.url = cls._normalize_url(url)
        obj.proxies = proxies
        obj = cls._initialize_attrs(obj)
        return obj

    _shared_docs['is_valid'] = (
        """Check whether the site has valid API.

        Returns
        -------
        is_valid : bool
        """)

    @Appender(_shared_docs['is_valid'])
    def is_valid(self):
        return True

    _shared_docs['get'] = (
        """Get resource by resource_id.

        Parameters
        ----------
        resource_id : str
            id to specify resource

        Returns
        -------
        result : %(resource_klass)s
        """)

    @Appender(_shared_docs['get'] % _base_doc_kwargs)
    def get(self, name):
        raise NotImplementedError

    _shared_docs['search'] = (
        """Search resources by search_string.

        Parameters
        ----------
        search storing : str
            keyword to search

        Returns
        -------
        result : list of %(resource_klass)s
        """)

    @Appender(_shared_docs['search'] % _base_doc_kwargs)
    def search(self, serch_string):
        raise NotImplementedError

    @property
    def datasets(self):
        raise NotImplementedError



# pylint: disable-msg=E1101,W0613,W0603
import pyopendata.io
from pyopendata.base import DataStore, DataResource
from pyopendata.ckan import CKANStore, CKANPackage, CKANResource
from pyopendata.eurostat import EurostatStore, EurostatResource
from pyopendata.oecd import OECDStore, OECDResource
from pyopendata.undata import UNdataStore, UNdataResource
from pyopendata.worldbank import WorldBankStore, WorldBankResource

from pyopendata.version import version as __version__


# pylint: disable-msg=E1101,W0613,W0603

from __future__ import unicode_literals

import pandas as pd
import numpy as np
from pandas.compat import u, bytes_to_str, range, iterkeys, iteritems

from pyopendata.base import DataStore, DataResource
import pyopendata.io.sdmx as sdmx



class UNdataStore(DataStore):
    """
    Storage class to read UNdata data
    """

    _url = 'http://data.un.org/Handlers'
    _cache_attrs = ['_datasets']

    # reference
    # http://blog.overcognition.com/?p=422

    def is_valid(self):
        """
        Check whether the site has valid API.

        Returns
        -------
        is_valid : bool
        """
        return True

    def get(self, data_id):
        resource = UNdataResource(id=data_id)
        return resource

    @property
    def datasets(self):
        if self._datasets is None:
            response = self._requests_get('/ExplorerHandler.ashx?t=marts')
            # format is json, but `Nodes` is not double-quoted
            # Thus, unable to parse by response.json()

            # replace 1st Nodes to "Nodes"
            content = bytes_to_str(response.content)
            content = content.replace(str('Nodes'), str('"Nodes"'), 1)
            import json
            result = json.loads(content)
            nodes = result['Nodes']

            # import html.parser
            # import HTMLParser
            # parser = HTMLParser.HTMLParser()
            def delabel(node):
                # print(node['label'])
                # print(parser.feed(node['label']))
                return node

            nodes = [delabel(n) for n in nodes]
            datasets = pd.DataFrame(result['Nodes'])

            self._datasets = []
            # for dataflow in root.iter(sdmx._STRUCTURE + 'Dataflow'):
            #     name = sdmx._get_english_name(dataflow)
            #     id = dataflow.get('id')
            #     resource = EurostatResource(name=name, id=id)
            #     self._datasets.append(resource)
        return self._datasets


class UNdataResource(DataResource):

    def __init__(self, **kwargs):
        DataResource.__init__(self, **kwargs)
        # query = 'DataMartId={0}&DataFilter={1}&Format=csv'.format(self.mart, self.code)

        self.url = UNdataStore._url + '/DocumentDownloadHandler.ashx?t=bin&id={0}'.format(self.id)

        # not sure which API is better
        # DownloadHandler.ashx?DataMartId=Comtrade&DataFilter=_l1Code%3a1&Format=csv
        # DocumentDownloadHandler.ashx?t=bin&id=164

    def _read(self, skiprows=[0, 1, 2, 3], **kwargs):
        try:
            data = self._read_raw()
            result = pd.read_excel(data, skiprows=skiprows, **kwargs)
        except Exception:
            result = pd.read_excel(self.url, skiprows=skiprows, **kwargs)
        return result





# pylint: disable-msg=E1101,W0613,W0603

from __future__ import unicode_literals

import numpy as np
import pandas as pd
from pandas.compat import iterkeys
from pandas.util.decorators import Appender

from pyopendata.base import DataStore, DataResource, _shared_docs
from pyopendata.io import read_jsdmx


_oecd_doc_kwargs = dict(resource_klass='OECDResource')


class OECDStore(DataStore):
    """
    Storage class to read OECD data
    """
    # http://www.oecd.org/about/membersandpartners/list-oecd-member-countries.htm

    _url = 'http://stats.oecd.org/SDMX-JSON/data'
    _cache_attrs = ['_datasets']

    _countries = {'AUS': 'AUSTRALIA',
                  'AUT': 'AUSTRIA',
                  'BEL': 'BELGIUM',
                  'CAN': 'CANADA',
                  'CHL': 'CHILE',
                  'CZE': 'CZECH REPUBLIC',
                  'DNK': 'DENMARK',
                  'EST': 'ESTONIA',
                  'FIN': 'FINLAND',
                  'FRA': 'FRANCE',
                  'DEU': 'GERMANY',
                  'GRC': 'GREECE',
                  'HUN': 'HUNGARY',
                  'ISL': 'ICELAND',
                  'IRL': 'IRELAND',
                  'ISR': 'ISRAEL',
                  'ITA': 'ITALY',
                  'JPN': 'JAPAN',
                  'KOR': 'KOREA',
                  'LUX': 'LUXEMBOURG',
                  'MEX': 'MEXICO',
                  'NLD': 'NETHERLANDS',
                  'NZL': 'NEW ZEALAND',
                  'NOR': 'NORWAY',
                  'POL': 'POLAND',
                  'PRT': 'PORTUGAL',
                  'SVK': 'SLOVAK REPUBLIC',
                  'SVN': 'SLOVENIA',
                  'ESP': 'SPAIN',
                  'SWE': 'SWEDEN',
                  'CHE': 'SWITZERLAND',
                  'TUR': 'TURKEY',
                  'GBR': 'UNITED KINGDOM',
                  'USA': 'UNITED STATES',
                  'OECD': 'OECD'}

    # def __init__(self, **kwargs):
    #     DataStore.__init__(self, url=self._url, **kwargs)

    @property
    def _target_countries(self):
        return '+'.join(list(iterkeys(self._countries)))

    @Appender(_shared_docs['get'] % _oecd_doc_kwargs)
    def get(self, resource_id):
        url = self.url + '/{0}/{1}/OECD?'.format(resource_id, self._target_countries)
        return OECDResource(id=resource_id, url=url)


class OECDResource(DataResource):

    def _read(self, **kwargs):
        data = self._requests_get().json()
        result = read_jsdmx(data)
        # There is data not be sorted by time
        result = result.sort_index()
        return result
